@@ -1,4 +1,4 @@
-/* $Id: fdm.c,v 1.180 2008/12/17 22:38:02 nicm Exp $ */
+/* $Id: fdm.c,v 1.185 2011/09/14 13:36:19 nicm Exp $ */
 
 /*
  * Copyright (c) 2006 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -44,9 +44,12 @@ const char		*malloc_options = "AFGJPRX";
 
 void			 sighandler(int);
 struct child		*check_children(struct children *, u_int *);
+int			 wait_children(
+			     struct children *, struct children *, int);
 
 struct conf		 conf;
 
+volatile sig_atomic_t	 sigchld;
 volatile sig_atomic_t	 sigusr1;
 volatile sig_atomic_t	 sigint;
 volatile sig_atomic_t	 sigterm;
@@ -67,6 +70,9 @@ sighandler(int sig)
 	case SIGTERM:
 		sigterm = 1;
 		break;
+	case SIGCHLD:
+		sigchld = 1;
+		break;
 	}
 }
 
@@ -179,7 +185,82 @@ check_children(struct children *children, u_int *idx)
 			return (child);
 	}
 	return (NULL);
-}	
+}
+
+/* Wait for a child and deal with its exit. */
+int
+wait_children(
+    struct children *children, struct children *dead_children, int no_hang)
+{
+	struct child	*child, *child2;
+	pid_t		 pid;
+	int		 status, flags, retcode = 0;
+	u_int		 i, j;
+
+	flags = no_hang ? WNOHANG : 0;
+	for (;;) {
+		log_debug3("parent: waiting for children");
+		/* Wait for a child. */
+		switch (pid = waitpid(WAIT_ANY, &status, flags)) {
+		case 0:
+			return (0);
+		case -1:
+			if (errno == ECHILD)
+				return (0);
+			fatal("waitpid failed");
+		}
+
+		/* Handle the exit status. */
+		if (WIFSIGNALED(status)) {
+			retcode = 1;
+			log_debug2("parent: child %ld got signal %d",
+			    (long) pid, WTERMSIG(status));
+		} else if (!WIFEXITED(status)) {
+			retcode = 1;
+			log_debug2("parent: child %ld exited badly",
+			    (long) pid);
+		} else {
+			if (WEXITSTATUS(status) != 0)
+				retcode = 1;
+			log_debug2("parent: child %ld returned %d",
+			    (long) pid, WEXITSTATUS(status));
+		}
+
+		/* Find this child. */
+		child = NULL;
+		for (i = 0; i < ARRAY_LENGTH(children); i++) {
+			child = ARRAY_ITEM(children, i);
+			if (pid == child->pid)
+				break;
+		}
+		if (i == ARRAY_LENGTH(children)) {
+			log_debug2("parent: unidentified child %ld",
+			    (long) pid);
+			continue;
+		}
+
+		if (child->io != NULL) {
+			io_close(child->io);
+			io_free(child->io);
+			child->io = NULL;
+		}
+		ARRAY_REMOVE(children, i);
+		ARRAY_ADD(dead_children, child);
+
+		/* If this child was the parent of any others, kill them too. */
+		for (j = 0; j < ARRAY_LENGTH(children); j++) {
+			child2 = ARRAY_ITEM(children, j);
+			if (child2->parent != child)
+				continue;
+
+			log_debug2("parent: child %ld died: killing %ld",
+			    (long) child->pid, (long) child2->pid);
+			kill(child2->pid, SIGTERM);
+		}
+	}
+
+	return (retcode);
+}
 
 __dead void
 usage(void)
@@ -198,7 +279,6 @@ main(int argc, char **argv)
 	enum fdmop       op = FDMOP_NONE;
 	const char	*proxy = NULL, *s;
 	char		 tmp[BUFSIZ], *ptr, *lock = NULL, *user, *home = NULL;
-	long		 n;
 	struct utsname	 un;
 	struct passwd	*pw;
 	struct stat	 sb;
@@ -207,8 +287,8 @@ main(int argc, char **argv)
 	TAILQ_HEAD(, account) actaq; /* active accounts */
 	pid_t		 pid;
 	struct children	 children, dead_children;
-	struct child	*child, *child2;
-	struct io       *rio;
+	struct child	*child;
+	struct io       *dead_io;
 	struct iolist	 iol;
 	double		 tim;
 	struct sigaction act;
@@ -344,7 +424,7 @@ main(int argc, char **argv)
 	/* Find invoking user's details. */
 	if ((pw = getpwuid(getuid())) == NULL) {
 		log_warnx("unknown user: %lu", (u_long) geteuid());
-		exit(1); 
+		exit(1);
 	}
  	user = xstrdup(pw->pw_name);
 	if (home != NULL && *home != '\0')
@@ -401,7 +481,7 @@ main(int argc, char **argv)
 		ud = user_lookup(conf.def_user, conf.user_order);
 		if (ud == NULL) {
 			log_warnx("unknown user: %s", conf.def_user);
-			exit(1); 
+			exit(1);
 		}
 		user_free(ud);
 	}
@@ -409,7 +489,7 @@ main(int argc, char **argv)
 		ud = user_lookup(conf.cmd_user, conf.user_order);
 		if (ud == NULL) {
 			log_warnx("unknown user: %s", conf.cmd_user);
-			exit(1); 
+			exit(1);
 		}
 		user_free(ud);
 	}
@@ -576,13 +656,12 @@ main(int argc, char **argv)
 	sigaddset(&act.sa_mask, SIGUSR1);
 	sigaddset(&act.sa_mask, SIGINT);
 	sigaddset(&act.sa_mask, SIGTERM);
+	sigaddset(&act.sa_mask, SIGCHLD);
 	act.sa_flags = SA_RESTART;
 
 	act.sa_handler = SIG_IGN;
 	if (sigaction(SIGPIPE, &act, NULL) < 0)
 		fatal("sigaction failed");
-	if (sigaction(SIGUSR1, &act, NULL) < 0)
-		fatal("sigaction failed");
 	if (sigaction(SIGUSR2, &act, NULL) < 0)
 		fatal("sigaction failed");
 
@@ -597,6 +676,8 @@ main(int argc, char **argv)
 		fatal("sigaction failed");
 	if (sigaction(SIGTERM, &act, NULL) < 0)
 		fatal("sigaction failed");
+	if (sigaction(SIGCHLD, &act, NULL) < 0)
+		fatal("sigaction failed");
 
 	/* Check lock file. */
 	lock = conf.lock_file;
@@ -642,7 +723,7 @@ main(int argc, char **argv)
 	ARRAY_INIT(&children);
 	ARRAY_INIT(&dead_children);
 
-#ifndef NO_SETPROCTITLE
+#ifdef HAVE_SETPROCTITLE
 	setproctitle("parent");
 #endif
 	log_debug2("parent: started, pid is %ld", (long) getpid());
@@ -653,7 +734,7 @@ main(int argc, char **argv)
 	while (!TAILQ_EMPTY(&actaq) || ARRAY_LENGTH(&children) != 0) {
 		log_debug2("parent: %u children, %u dead children",
 		    ARRAY_LENGTH(&children), ARRAY_LENGTH(&dead_children));
-		
+
 		/* Stop on signal. */
 		if (sigint || sigterm)
 			break;
@@ -663,35 +744,43 @@ main(int argc, char **argv)
 		    ARRAY_LENGTH(&children) < (u_int) conf.max_accts)) {
 			a = TAILQ_FIRST(&actaq);
 			TAILQ_REMOVE(&actaq, a, active_entry);
-			
+
 			cfd = xmalloc(sizeof *cfd);
 			cfd->account = a;
 			cfd->op = op;
 			cfd->children = &children;
-			child = child_start(&children, 
-			    conf.child_uid, conf.child_gid, 
+			child = child_start(&children,
+			    conf.child_uid, conf.child_gid,
 			    child_fetch, parent_fetch, cfd, NULL);
-			log_debug2("parent: child %ld (%s) started", 
-			    (long) child->pid, a->name);	
+			log_debug2("parent: child %ld (%s) started",
+			    (long) child->pid, a->name);
 		}
 
-		/* Fill the io list. */
+		/* Check children and fill the io list. */
 		ARRAY_CLEAR(&iol);
 		for (i = 0; i < ARRAY_LENGTH(&children); i++) {
 			child = ARRAY_ITEM(&children, i);
-			ARRAY_ADD(&iol, child->io);
+			if (child->io != NULL)
+				ARRAY_ADD(&iol, child->io);
 		}
 
 		/* Poll the io list. */
-		n = io_polln(
-		    ARRAY_DATA(&iol), ARRAY_LENGTH(&iol), &rio, INFTIM, NULL);
-		switch (n) {
-		case -1:
-			fatalx("child socket error");
-		case 0:
-			fatalx("child socket closed");
+		if (ARRAY_LENGTH(&iol) != 0) {
+			switch (io_polln(ARRAY_DATA(&iol), ARRAY_LENGTH(&iol),
+			    &dead_io, INFTIM, NULL)) {
+			case -1:
+			case 0:
+				break;
+			default:
+				dead_io = NULL;
+				break;
+			}
+		} else {
+			/* No more children. Sleep until all are waited. */
+			if (wait_children(&children, &dead_children, 0) != 0)
+				res = 1;
 		}
-		
+
 		/* Check all children for pending privsep messages. */
 		while ((child = check_children(&children, &i)) != NULL) {
 			/* Handle this message. */
@@ -701,50 +790,34 @@ main(int argc, char **argv)
 			    "child %ld", msg.type, msg.id, (long) child->pid);
 			if (child->msg(child, &msg, &msgbuf) == 0)
 				continue;
-		
+
 			/* Child has said it is ready to exit, tell it to. */
+			log_debug2("parent: sending exit message to child %ld",
+			    (long) child->pid);
 			memset(&msg, 0, sizeof msg);
 			msg.type = MSG_EXIT;
 			if (privsep_send(child->io, &msg, NULL) != 0)
 				fatalx("privsep_send error");
-		
-			/* Wait for the child. */
-			if (waitpid(child->pid, &status, 0) == -1)
-				fatal("waitpid failed");
-			if (WIFSIGNALED(status)) {
-				res = 1;
-				log_debug2("parent: child %ld got signal %d",
-				    (long) child->pid, WTERMSIG(status));
-			} else if (!WIFEXITED(status)) {
-				res = 1;
-				log_debug2("parent: child %ld exited badly",
-				    (long) child->pid);
-			} else {
-				if (WEXITSTATUS(status) != 0)
-					res = 1;
-				log_debug2("parent: child %ld returned %d",
-				    (long) child->pid, WEXITSTATUS(status));
-			}
-			
-			io_close(child->io);
-			io_free(child->io);
-			child->io = NULL;
-			
-			ARRAY_REMOVE(&children, i);
-			ARRAY_ADD(&dead_children, child);
-			
-			/*
-			 * If this child was the parent of any others, kill
-			 * them too.
-			 */
+		}
+
+		/* Collect any dead children. */
+		if (sigchld && wait_children(&children, &dead_children, 1) != 0)
+			res = 1;
+		sigchld = 0;
+
+		/* Close dead buffers (no more data coming now). */
+		if (dead_io != NULL) {
 			for (i = 0; i < ARRAY_LENGTH(&children); i++) {
-				child2 = ARRAY_ITEM(&children, i);
-				if (child2->parent != child)
+				child = ARRAY_ITEM(&children, i);
+				if (dead_io != child->io)
 					continue;
-				
-				log_debug("parent: child %ld died: killing %ld",
-				    (long) child->pid, (long) child2->pid);
-				kill(child2->pid, SIGTERM);
+				log_debug2("parent: child %ld socket error",
+				    (long) child->pid);
+				kill(child->pid, SIGTERM);
+
+				io_close(child->io);
+				io_free(child->io);
+				child->io = NULL;
 			}
 		}
 	}
