@@ -1,4 +1,4 @@
-/* $Id: fdm.h,v 1.344 2008/09/07 21:51:41 nicm Exp $ */
+/* $Id: fdm.h,v 1.351 2011/10/10 13:36:29 nicm Exp $ */
 
 /*
  * Copyright (c) 2006 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -19,17 +19,19 @@
 #ifndef FDM_H
 #define FDM_H
 
+#include "config.h"
+
 #include <sys/param.h>
 #include <sys/cdefs.h>
 #include <sys/stat.h>
 
-#ifndef NO_QUEUE_H
+#ifdef HAVE_QUEUE_H
 #include <sys/queue.h>
 #else
 #include "compat/queue.h"
 #endif
 
-#ifndef NO_TREE_H
+#ifdef HAVE_TREE_H
 #include <sys/tree.h>
 #else
 #include "compat/tree.h"
@@ -39,6 +41,9 @@
 #include <signal.h>
 #include <stdarg.h>
 #include <stdint.h>
+#ifndef _PUBLIC_
+#define _PUBLIC_
+#endif
 #include <tdb.h>
 #include <regex.h>
 
@@ -377,6 +382,7 @@ struct mail_ctx {
 	struct io			*io;
 	struct mail			*mail;
 
+	u_int				 ruleidx;
 	struct rule			*rule;
 	ARRAY_DECL(, struct rule *)	 stack;
 	struct expritem			*expritem;
@@ -690,25 +696,25 @@ struct file {
 };
 ARRAY_DECL(files, struct file *);
 
-#ifdef NO_SETRESUID
+#ifndef HAVE_SETRESUID
 #define setresuid(r, e, s) setreuid(r, e)
 #endif
 
-#ifdef NO_SETRESGID
+#ifndef HAVE_SETRESGID
 #define setresgid(r, e, s) setregid(r, e)
 #endif
 
-#ifdef NO_STRTONUM
+#ifndef HAVE_STRTONUM
 /* strtonum.c */
 long long	 strtonum(const char *, long long, long long, const char **);
 #endif
 
-#ifdef NO_STRLCPY
+#ifndef HAVE_STRLCPY
 /* strlcpy.c */
 size_t	 	 strlcpy(char *, const char *, size_t);
 #endif
 
-#ifdef NO_STRLCAT
+#ifndef HAVE_STRLCAT
 /* strlcat.c */
 size_t	 	 strlcat(char *, const char *, size_t);
 #endif
@@ -745,6 +751,7 @@ struct actions	*match_actions(const char *);
 struct macro 	*extract_macro(char *);
 struct macro	*find_macro(const char *);
 void		 find_netrc(const char *, char **, char **);
+int		 find_netrc1(const char *, char **, char **, char **);
 void		 free_account(struct account *);
 void		 free_action(struct action *);
 void		 free_actitem(struct actitem *);
@@ -816,7 +823,7 @@ void		 cmd_free(struct cmd *);
 /* child.c */
 int		 child_fork(void);
 __dead void	 child_exit(int);
-struct child 	*child_start(struct children *, uid_t, gid_t, 
+struct child 	*child_start(struct children *, uid_t, gid_t,
     		     int (*)(struct child *, struct io *),
     		     int (*)(struct child *, struct msg *, struct msgbuf *),
 		     void *, struct child *);
@@ -865,6 +872,7 @@ int		 xmkpath(const char *, uid_t, gid_t, mode_t);
 const char 	*checkmode(struct stat *, mode_t);
 const char 	*checkowner(struct stat *, uid_t);
 const char 	*checkgroup(struct stat *, gid_t);
+int		 safemove(const char *, const char *);
 
 /* mail.c */
 int		 mail_open(struct mail *, size_t);
