@@ -1,4 +1,4 @@
-/* $Id: fetch-maildir.c,v 1.87 2007/12/12 08:05:33 nicm Exp $ */
+/* $Id: fetch-maildir.c,v 1.93 2011/09/14 13:36:19 nicm Exp $ */
 
 /*
  * Copyright (c) 2006 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -95,24 +95,28 @@ fetch_maildir_makepaths(struct account *a)
 			ARRAY_ADD(data->paths, path);
 			if (stat(path, &sb) != 0) {
 				log_warn("%s: %s", a->name, path);
-				goto error;
+				ARRAY_TRUNC(data->paths, 1);
+				continue;
 			}
 			if (!S_ISDIR(sb.st_mode)) {
 				errno = ENOTDIR;
 				log_warn("%s: %s", a->name, path);
-				goto error;
+				ARRAY_TRUNC(data->paths, 1);
+				continue;
 			}
 
 			xasprintf(&path, "%s/new", g.gl_pathv[j]);
 			ARRAY_ADD(data->paths, path);
 			if (stat(path, &sb) != 0) {
 				log_warn("%s", path);
-				goto error;
+				ARRAY_TRUNC(data->paths, 2);
+				continue;
 			}
 			if (!S_ISDIR(sb.st_mode)) {
 				errno = ENOTDIR;
 				log_warn("%s", path);
-				goto error;
+				ARRAY_TRUNC(data->paths, 2);
+				continue;
 			}
 		}
 
@@ -133,10 +137,14 @@ fetch_maildir_freepaths(struct account *a)
 	struct fetch_maildir_data	*data = a->data;
 	u_int			 	 i;
 
+	if (data->paths == NULL)
+		return;
+
 	for (i = 0; i < ARRAY_LENGTH(data->paths); i++)
 		xfree(ARRAY_ITEM(data->paths, i));
 
 	ARRAY_FREEALL(data->paths);
+	data->paths = NULL;
 }
 
 /* Count maildir total. */
@@ -199,12 +207,13 @@ fetch_maildir_poll(struct account *a)
 int
 fetch_maildir_commit(struct account *a, struct mail *m)
 {
+	struct fetch_maildir_data	*data = a->data;
 	struct fetch_maildir_mail	*aux;
 
 	aux = m->auxdata;
-	if (m->decision == DECISION_DROP && unlink(aux->path) != 0) {
-		log_warn("%s: %s: unlink", a->name, aux->path);
-		return (FETCH_ERROR);
+	if (m->decision == DECISION_DROP) {
+		/* Add mail to the unlink list. */
+		ARRAY_ADD(&data->unlinklist, xstrdup(aux->path));
 	}
 
 	return (FETCH_AGAIN);
@@ -215,6 +224,11 @@ void
 fetch_maildir_abort(struct account *a)
 {
 	struct fetch_maildir_data	*data = a->data;
+	u_int				 i;
+
+	for (i = 0; i < ARRAY_LENGTH(&data->unlinklist); i++)
+		xfree(ARRAY_ITEM(&data->unlinklist, i));
+	ARRAY_FREE(&data->unlinklist);
 
 	if (data->dirp != NULL)
 		closedir(data->dirp);
@@ -240,11 +254,12 @@ fetch_maildir_state_init(struct account *a, struct fetch_ctx *fctx)
 		return (FETCH_ERROR);
 	if (ARRAY_EMPTY(data->paths)) {
 		log_warnx("%s: no maildirs found", a->name);
-		return (-1);
+		return (FETCH_ERROR);
 	}
 
 	data->index = 0;
 	data->dirp = NULL;
+	ARRAY_INIT(&data->unlinklist);
 
 	/* Poll counts mails and exits. */
 	if (fctx->flags & FETCH_POLL) {
@@ -255,7 +270,7 @@ fetch_maildir_state_init(struct account *a, struct fetch_ctx *fctx)
 	}
 
 	fctx->state = fetch_maildir_state_open;
-	return (0);
+	return (FETCH_AGAIN);
 }
 
 /* Next state. Move to next path. */
@@ -263,6 +278,19 @@ int
 fetch_maildir_state_next(struct account *a, struct fetch_ctx *fctx)
 {
 	struct fetch_maildir_data	*data = a->data;
+	char				*path;
+	u_int				 i;
+
+	for (i = 0; i < ARRAY_LENGTH(&data->unlinklist); i++) {
+		path = ARRAY_ITEM(&data->unlinklist, i);
+		log_debug2("%s: unlinking: %s", a->name, path);
+		if (unlink(path) != 0) {
+			log_warn("%s: %s: unlink", a->name, path);
+			return (FETCH_ERROR);
+		}
+		xfree(path);
+	}
+	ARRAY_FREE(&data->unlinklist);
 
 	if (data->index < ARRAY_LENGTH(data->paths))
 		data->index++;
@@ -366,6 +394,8 @@ restart:
 	maildir = xbasename(xdirname(path));
 	default_tags(&m->tags, maildir);
 	add_tag(&m->tags, "maildir", "%s", maildir);
+	add_tag(&m->tags, "maildir_path", "%s", xdirname(path));
+	add_tag(&m->tags, "maildir_file", "%s", path);
 
 	/* Add aux data. */
 	aux = xmalloc(sizeof *aux);
