@@ -1,4 +1,4 @@
-/* $Id: match-matched.c,v 1.4 2006/11/28 18:14:33 nicm Exp $ */
+/* $Id: match-matched.c,v 1.8 2007/03/19 20:04:48 nicm Exp $ */
 
 /*
  * Copyright (c) 2006 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -21,22 +21,27 @@
 #include <string.h>
 
 #include "fdm.h"
+#include "match.h"
 
-int	matched_match(struct match_ctx *, struct expritem *);
-char   *matched_desc(struct expritem *);
+int	match_matched_match(struct mail_ctx *, struct expritem *);
+void	match_matched_desc(struct expritem *, char *, size_t);
 
-struct match match_matched = { matched_match, matched_desc };
+struct match match_matched = {
+	"matched",
+	match_matched_match,
+	match_matched_desc
+};
 
 int
-matched_match(struct match_ctx *mctx, unused struct expritem *ei)
+match_matched_match(struct mail_ctx *mctx, unused struct expritem *ei)
 {
 	if (mctx->matched)
 		return (MATCH_TRUE);
 	return (MATCH_FALSE);
 }
 
-char *
-matched_desc(unused struct expritem *ei)
+void
+match_matched_desc(unused struct expritem *ei, char *buf, size_t len)
 {
-	return (xstrdup("matched"));
+	strlcpy(buf, "matched", len);
 }
