@@ -1,4 +1,4 @@
-/* $Id: match-regexp.c,v 1.20 2007/03/19 20:04:48 nicm Exp $ */
+/* $Id: match-regexp.c,v 1.22 2007/07/22 14:29:25 nicm Exp $ */
 
 /*
  * Copyright (c) 2006 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -46,17 +46,18 @@ match_regexp_match(struct mail_ctx *mctx, struct expritem *ei)
 	eo = m->size;
 	switch (data->area) {
 	case AREA_HEADERS:
-		if (m->body != -1)
-			eo = m->body;
+		if (m->body == 0)
+			return (MATCH_FALSE);
+		eo = m->body;
 		break;
 	case AREA_BODY:
-		if (m->body == -1)
-			return (MATCH_FALSE);
 		so = m->body;
 		break;
 	case AREA_ANY:
 		break;
 	}
+	log_debug3("%s: matching from %zu to %zu (size=%zu, body=%zu)", a->name,
+	    so, eo, m->size, m->body);
 
 	res = re_block(&data->re, m->data + so, eo - so, &m->rml, &cause);
 	if (res == -1) {
